-----------------------------------------------
-- Licensed Materials - Property of IBM
--
-- 5724-R31, 5655-S30
--
-- (C) Copyright IBM Corp. 2006, 2008 All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
------------------------------------------------



CREATE TABLE w_lit_double (
    id int IDENTITY PRIMARY KEY CLUSTERED,
    litval float UNIQUE NOT NULL
);

CREATE NONCLUSTERED INDEX idx_obj_dbl ON w_lit_double (litval);

CREATE TABLE w_lit_float (
    id int IDENTITY PRIMARY KEY CLUSTERED,
    litval real UNIQUE NOT NULL
);

CREATE NONCLUSTERED INDEX idx_obj_flt ON w_lit_float (litval);

CREATE TABLE w_lit_long (
    id int IDENTITY PRIMARY KEY CLUSTERED,
    litval bigint UNIQUE NOT NULL
);

CREATE NONCLUSTERED INDEX idx_obj_long ON w_lit_long (litval);

CREATE TABLE w_namespace (
    id int IDENTITY PRIMARY KEY CLUSTERED, 
    namespace nvarchar(254) COLLATE Latin1_General_CS_AI UNIQUE NOT NULL
);

CREATE UNIQUE INDEX idx_namespace ON w_namespace
    (namespace ASC) ;

CREATE TABLE w_obj_lit_any (
    id int IDENTITY PRIMARY KEY CLUSTERED,
    large image NULL,
    hash nvarchar(40) COLLATE Latin1_General_CS_AI NULL,
    type_uri nvarchar(254) COLLATE Latin1_General_CS_AI NOT NULL
);

CREATE NONCLUSTERED INDEX idx_obj_any ON w_obj_lit_any (type_uri);

CREATE TABLE w_obj_lit_date (
    id int IDENTITY PRIMARY KEY CLUSTERED,
    litval datetime UNIQUE NOT NULL
);

CREATE NONCLUSTERED INDEX idx_obj_date ON w_obj_lit_date (litval);

CREATE TABLE w_obj_lit_datetime (
    id int IDENTITY PRIMARY KEY CLUSTERED,
    litval datetime UNIQUE NOT NULL
);

CREATE NONCLUSTERED INDEX idx_obj_time ON w_obj_lit_datetime (litval);

CREATE TABLE w_obj_lit_string (
    id int IDENTITY PRIMARY KEY CLUSTERED,
    large image NULL,
    litval nvarchar(1024) COLLATE Latin1_General_CS_AI NULL,
    hash nvarchar(40) COLLATE Latin1_General_CS_AI NULL
);

CREATE NONCLUSTERED INDEX IDX_OBJ_STR ON w_obj_lit_string
    (hash);

CREATE TABLE w_uri (
    id int IDENTITY PRIMARY KEY CLUSTERED,
    uri nvarchar(254) COLLATE Latin1_General_CS_AI UNIQUE NOT NULL,
    namespace_id int NOT NULL
);

CREATE UNIQUE INDEX idx_uri ON w_uri (uri) ;
CREATE UNIQUE INDEX idx_uri_by_ns ON w_uri (namespace_id,id);

CREATE TABLE w_statement (
    id int IDENTITY PRIMARY KEY CLUSTERED,
    version_from int NOT NULL,
    version_to int NOT NULL,
    subj_id int NOT NULL,
    pred_id int NOT NULL,
    obj_id int NOT NULL,
    obj_typ_cd int NOT NULL,
    partition_id int NOT NULL
);


CREATE NONCLUSTERED INDEX IDX_SMT_BY_SBJ ON w_statement (subj_id, version_from, version_to);
CREATE NONCLUSTERED INDEX IDX_SMT_BY_FVR ON w_statement (version_from);
CREATE NONCLUSTERED INDEX IDX_SBJ_BY_PRP ON w_statement (pred_id, obj_id, partition_id);
CREATE NONCLUSTERED INDEX IDX_SBJ_BY_VAL ON w_statement (obj_id, subj_id);
CREATE NONCLUSTERED INDEX IDX_VAL_BY_PRP ON w_statement (subj_id, pred_id);

CREATE TABLE w_version (
    id int NOT NULL,
    username nvarchar(64) COLLATE Latin1_General_CS_AI NOT NULL,
    change_time datetime NOT NULL,
    cl_gid nvarchar(36) COLLATE Latin1_General_CS_AI NULL,
    cl_lid nvarchar(10) COLLATE Latin1_General_CS_AI NULL,
    schema_ns_id int NULL,
    schema_rev int NULL,
    partition_id int NOT NULL
);

ALTER TABLE w_version ADD CONSTRAINT pk_version PRIMARY KEY CLUSTERED (id, partition_id);

CREATE NONCLUSTERED INDEX IDX_VER_SCHEMA ON w_version (schema_ns_id);

CREATE TABLE w_artifact_blob (
 id NVARCHAR(254) COLLATE Latin1_General_CS_AI NOT NULL, 
 content IMAGE, 
 deleted SMALLINT,
 PRIMARY KEY(id)
);

ALTER TABLE w_uri
    ADD CONSTRAINT W_URI_FK_NAMESP FOREIGN KEY (namespace_id) 
    REFERENCES w_namespace (id) 
    ON UPDATE NO ACTION
    ON DELETE NO ACTION;

ALTER TABLE w_version
    ADD CONSTRAINT W_VER_FK_NAMESP FOREIGN KEY (schema_ns_id) 
    REFERENCES w_namespace (id) 
    ON UPDATE NO ACTION
    ON DELETE NO ACTION;
    
CREATE TABLE w_locale (
    id int NOT NULL,
    locale nvarchar(8) COLLATE Latin1_General_CS_AI NOT NULL,
    PRIMARY KEY(id)
);

ALTER TABLE w_locale ADD CONSTRAINT unique_locale UNIQUE (locale);

INSERT INTO w_locale VALUES (1000, '');
INSERT INTO w_locale VALUES (1001, 'en');
INSERT INTO w_locale VALUES (1002, 'en-us');
INSERT INTO w_locale VALUES (1003, 'en-uk');
INSERT INTO w_locale VALUES (1004, 'en-gb');
INSERT INTO w_locale VALUES (1032, 'cs');
INSERT INTO w_locale VALUES (1064, 'es');
INSERT INTO w_locale VALUES (1096, 'de');
INSERT INTO w_locale VALUES (1128, 'de-de');
INSERT INTO w_locale VALUES (1129, 'fr');
INSERT INTO w_locale VALUES (1160, 'fr-ca-sk');
INSERT INTO w_locale VALUES (1161, 'hu');
INSERT INTO w_locale VALUES (1192, 'it');
INSERT INTO w_locale VALUES (1224, 'ja');
INSERT INTO w_locale VALUES (1256, 'ko');
INSERT INTO w_locale VALUES (1288, 'pl');
INSERT INTO w_locale VALUES (1320, 'pt-br');
INSERT INTO w_locale VALUES (1352, 'ru');
INSERT INTO w_locale VALUES (1384, 'zh');
INSERT INTO w_locale VALUES (1416, 'zh-tw');
INSERT INTO w_locale VALUES (1448, 'el');


-- Pattern tables for query optimization

CREATE TABLE w_subj_pred_index (
    subj_id int NOT NULL REFERENCES w_uri (id),
    predicate_map VARBINARY(1024),
    version_from int NOT NULL,
    version_to int NOT NULL,
    partition_id int NOT NULL REFERENCES w_uri (id),
    PRIMARY KEY CLUSTERED (subj_id, version_from, version_to, partition_id)
);

CREATE TABLE w_subj_obj_index (
    subj_id int NOT NULL REFERENCES w_uri (id),
    object_map VARBINARY(1024),
    obj_typ_cd int NOT NULL,
    version_from int NOT NULL,
    version_to int NOT NULL,
    partition_id int NOT NULL REFERENCES w_uri (id),
    PRIMARY KEY CLUSTERED (subj_id, obj_typ_cd, version_from, version_to, partition_id)
);

CREATE TABLE w_pred_subj_index (
    pred_id int NOT NULL REFERENCES w_uri (id),
    subject_map VARBINARY(1024),
    version_from int NOT NULL,
    version_to int NOT NULL,
    partition_id int NOT NULL REFERENCES w_uri (id),
    PRIMARY KEY CLUSTERED (pred_id, version_from, version_to, partition_id)
);

CREATE TABLE w_pred_obj_index (
    pred_id int NOT NULL REFERENCES w_uri (id),
    object_map VARBINARY(1024),
    obj_typ_cd int NOT NULL,
    version_from int NOT NULL,
    version_to int NOT NULL,
    partition_id int NOT NULL REFERENCES w_uri (id),
    PRIMARY KEY CLUSTERED (pred_id, obj_typ_cd, version_from, version_to, partition_id)
);

CREATE TABLE w_obj_subj_index (
    obj_id int NOT NULL,
    obj_typ_cd int NOT NULL,
    subject_map VARBINARY(1024),
    version_from int NOT NULL,
    version_to int NOT NULL,
    partition_id int NOT NULL REFERENCES w_uri (id),
    PRIMARY KEY CLUSTERED (obj_id, obj_typ_cd, version_from, version_to, partition_id)
);

CREATE TABLE w_obj_pred_index (
    obj_id int NOT NULL,
    obj_typ_cd int NOT NULL,
    predicate_map VARBINARY(1024),
    version_from int NOT NULL,
    version_to int NOT NULL,
    partition_id int NOT NULL REFERENCES w_uri (id),
    PRIMARY KEY CLUSTERED (obj_id, obj_typ_cd, version_from, version_to, partition_id)
);



CREATE TABLE w_dbversion(
    subsystem nvarchar(8) PRIMARY KEY CLUSTERED, 
    cur_version int NOT NULL 
);
