import sys
from com.ibm.j2ca.migration.wpsruntime.base import WPSMigrationUtil
from java.lang import System

# Retrieve the value of WPS HOME from system environment property
wpsRootPath=WPSMigrationUtil.getSystemProperty("was.install.root")
# Define the script path
fileSeparator=WPSMigrationUtil.getFileSeparator()
scriptsPath=wpsRootPath+fileSeparator+"util"+fileSeparator+"adapterMigration"+fileSeparator
sys.path.append(scriptsPath)

import printHelper as PrintHelper

# Set the environment property value: WPS_HOME
PrintHelper.printInfo("WPS_HOME = "+wpsRootPath)
WPSMigrationUtil.setWPSHome(wpsRootPath)
# Set the environment property value for new adapter RAR path
# newRARFolder=wpsRootPath+fileSeparator+"installableApps"+fileSeparator
# WPSMigrationUtil.setNewRARDir(newRARFolder)
# Set the environment property value for migration temp folder
tempFolder=WPSMigrationUtil.getSystemProperty("bpm.upgrade.backup.dir")
WPSMigrationUtil.setTempDir(tempFolder)
lineSeparator = System.getProperty("line.separator")

import findEmbeddedRAApplicationsForVersionConflict as FindEmbededRAAppsForVersionConflict
import findStandaloneRAApplicationsForVersionConflict as FindStandAloneRAAppsForVersionConflict
import updateEmbededRARForVersionConflict as UpdateEmbededRARForVersionConflict
import updateStandaloneRARForVersionConflict as UpdateStandAloneRARForVersionConflict
import updateClusterStandaloneRARForVersionConflict as UpdateClusterStandAloneRARForVersionConflict
import migrateApp as MigrateApp

#-------------------------------------------------------------------------------------------
#  Sub modules for embeded adapters/applications migration
#-------------------------------------------------------------------------------------------
def migrateEmbededRAApps(configurationFile):
	embededRAApps = FindEmbededRAAppsForVersionConflict.findEmbeddedRAApps()
	#PrintHelper.printInfo(embededRAApps)
	PrintHelper.printInfo("----Start embedded adapter/application migration...")
	for app in embededRAApps[:]:
		appName=app[0]
		PrintHelper.printInfo(appName)
		connectorModules=app[1]
		PrintHelper.printInfo(connectorModules)
		for connector in connectorModules[:]:
			adapterName=connector[0]
			adapterVersion=connector[1]
			adapterURI=connector[2]
			adapterID=connector[3]
			pureAdapterName = WPSMigrationUtil.getPureAdapterName(adapterName)
			PrintHelper.printInfo("adapterName = " + adapterName)
			PrintHelper.printInfo("pureAdapterName = " + pureAdapterName)
			#PrintHelper.printInfo(WPSMigrationUtil.isSupportedAdapterName(pureAdapterName))
			if (WPSMigrationUtil.isSupportedAdapterName(pureAdapterName)):
				# Export the application to ear file under the migration working directory
				migrationWorkDir = WPSMigrationUtil.getMigrationWorkDir()
				try:
					PrintHelper.printInfo("----Embedded application to be migrated = "+appName)
					exportEarFile = migrationWorkDir+appName+".ear"
					AdminApp.export(appName, exportEarFile)
					PrintHelper.printInfo("----Export app "+appName+" to file: "+exportEarFile)

					UpdateEmbededRARForVersionConflict.updateEmbededRAR(exportEarFile, appName, adapterName, adapterVersion, adapterURI, adapterID)
					MigrateApp.migrateApp(exportEarFile, appName, configurationFile)
					PrintHelper.printInfo("----Embedded application migration Done for app = "+appName)
				except:
					PrintHelper.printInfo("----FAIL: Embedded application migration failed for app = "+appName)
					PrintHelper.printInfo('----root cause: '+ sys.exc_type + sys.exc_value)
				# Delete the Work Dir
				if ( WPSMigrationUtil.toRemoveTempFiles() ):
					WPSMigrationUtil.deleteFile(migrationWorkDir)
		#endFor
	#endFor
#end def migrateEmbededRAApps

#-------------------------------------------------------------------------------------------
#  Sub modules for standalone adapters/applications migration
#-------------------------------------------------------------------------------------------
def migrateStandAloneRAApps(configurationFile, nodeName):
	raStandaloneRAAppList = FindStandAloneRAAppsForVersionConflict.findStandaloneRAApps(nodeName)

	for oneNode in raStandaloneRAAppList:
		toMigrateApps = []
		node = oneNode[0]
		PrintHelper.printInfo("----Node = " + node)
		adapters = oneNode[1]
		PrintHelper.printInfo(adapters)
		for oneAdapter in adapters:
			#PrintHelper.printInfo(oneAdapter)
			ra = oneAdapter[0]
			raN = oneAdapter[1]
			raV = oneAdapter[2]
			newRARPath = WPSMigrationUtil.getNewAdapterRARLoc(raN)
			PrintHelper.printInfo("----Update adapter "+raN+" from loc: "+newRARPath + "###")
			#raVNew = WPSMigrationUtil.getNewAdapterRARNameVersion()
			#if (raV<raVNew):
			#	PrintHelper.printInfo("####################"+ raVNew)
			PrintHelper.printInfo(ra)
			if ( WPSMigrationUtil.isSupportedAdapterName(raN)):
				nodename = node.split('(')[0]
				UpdateStandAloneRARForVersionConflict.updateStandAloneRAR(nodename, raN)
				CFs = oneAdapter[3]
				PrintHelper.printInfo(CFs)
				for oneCF in CFs:
					cfJNDI = oneCF[0]
					PrintHelper.printInfo(cfJNDI)
					apps_CF = oneCF[1]
					PrintHelper.printInfo(apps_CF)
					for oneApp_CF in apps_CF :
						appNames_CF = FindStandAloneRAAppsForVersionConflict.wsadminToList(oneApp_CF)
						for appName in appNames_CF:
							if ( appName not in toMigrateApps):
								toMigrateApps.append(appName)
					#endfor
				#endfor

				ASs = oneAdapter[4]
				PrintHelper.printInfo(ASs)
				for oneAS in ASs:
					asJNDI = oneAS[0]
					PrintHelper.printInfo(asJNDI)
					apps_AS = oneAS[1]
					PrintHelper.printInfo(apps_AS)
					for oneApp_AS in apps_AS :
						appNames_AS = FindStandAloneRAAppsForVersionConflict.wsadminToList(oneApp_AS)
						for appName in appNames_AS:
							if ( appName not in toMigrateApps):
								toMigrateApps.append(appName)
					#endfor
				#endfor
			#end if
		#endfor RA
		
		for appName in toMigrateApps:
			MigrateApp.migrateStandaloneApp(appName, configurationFile)
			PrintHelper.printInfo("----Application "+appName+" on node="+nodeName+" is migrated")

	#endfor
#end def migrateStandAloneRAApps

def migrateClusterStandaloneRAApps(configurationFile):
	#--------------------------------------------------------------
	# Will check the applications that reference to the adapter MCF/AS defined on clusters
	#--------------------------------------------------------------    
	clusterApps = FindStandAloneRAAppsForVersionConflict.findClusterStandaloneRAApps()
	#PrintHelper.printInfo(clusterApps)
	
	for oneCluster in clusterApps:
		toMigrateApps = []
		cluster = oneCluster[0]
		clusterRA_AppsList = oneCluster[1]
		for oneClusterRA in clusterRA_AppsList:
			# element description:
			# index value
			# 0	adapter ID
			# 1	adapter name
			# 2	adapter displayName
			# 3	adapter version
			# 4	application list that refer to adapter MCF
			# 5	application list that refer to adapter ActivationSpec
			clusterRA = oneClusterRA[0]
			clusterRAName = oneClusterRA[1]
			clusterRADisplayName = oneClusterRA[2]
			clusterRAVersion = oneClusterRA[3]
			if ( WPSMigrationUtil.isSupportedArtifact(clusterRADisplayName, clusterRAVersion)):
				# update the cluster level standalone adapter RAR
				UpdateClusterStandAloneRARForVersionConflict.updateClusterStandaloneRAR(cluster, clusterRA, clusterRADisplayName)
				PrintHelper.printInfo("----Cluster level standalone adapter "+clusterRAName+" is migrated")
			
				# find out the cluster level applications that refer to this adapter's MCF
				clusterRA_CFApps = oneClusterRA[4]
				for oneCF in clusterRA_CFApps:
					cfJNDI = oneCF[0]
					apps_CF = oneCF[1]
					for oneApp_CF in apps_CF :
						appNames_CF = FindStandAloneRAAppsForVersionConflict.wsadminToList(oneApp_CF)
						for appName in appNames_CF:
							if ( appName not in toMigrateApps):
								toMigrateApps.append(appName)
					#endfor
				#endfor
				
				# find out the cluster level applications that refer to this adapter's ActivationSpec
				clusterRA_ASApps = oneClusterRA[5]
				for oneAS in clusterRA_ASApps:
					asJNDI = oneAS[0]
					apps_AS = oneAS[1]
					for oneApp_AS in apps_AS :
						appNames_AS = FindStandAloneRAAppsForVersionConflict.wsadminToList(oneApp_AS)
						for appName in appNames_AS:
							if ( appName not in toMigrateApps):
								toMigrateApps.append(appName)
					#endfor
				#endfor
			#end if
		#end for -- RA

		# Migrate the cluster level applications that we found out above that refers to cluster level standalone adapter MCF/ActivationSpec
		for appName in toMigrateApps:
			MigrateApp.migrateStandaloneApp(appName, configurationFile)
			PrintHelper.printInfo("----Application "+appName+" on cluster is migrated")
		
	#endFor -- cluster
#end

def getNodeType():
	nodeType = WPSMigrationUtil.getSystemProperty("bpm.upgrade.node.type")
	if (nodeType==""):
		nodelist = AdminConfig.list('Node')
		nodeArr = FindStandAloneRAAppsForVersionConflict.wsadminToList(nodelist)
		nodeNum = len(nodeArr)
		if ( nodeNum > 1 ):
			nodeType = "dmgr"
		else:
			nodeType = "node"
	
	return nodeType
#end def getNodeType

def getClusterName():
	clusterName = WPSMigrationUtil.getSystemProperty("bpm.upgrade.cluster.name")
	return clusterName
# end def getClusterName

#-------------------------------------------------------------------------------------------
#  Main program
#-------------------------------------------------------------------------------------------
if __name__ == "__main__":
	PrintHelper.printInfo("*************************************************************")
	PrintHelper.printInfo("----Start to migrate all Adapters on Websphsre Process Sever!")
	configurationFile = scriptsPath+"adapterMigrationInfo.xml"
	PrintHelper.printInfo("----Loading configuration info from "+configurationFile)
	WPSMigrationUtil.getAdapterMigrationInfo(configurationFile)
	tempFolder = WPSMigrationUtil.getTempDir();
	PrintHelper.printInfo("Temp Folder = "+tempFolder)
	if (WPSMigrationUtil.validateTempFolder()):
		PrintHelper.printInfo("----migrationTempDir property in adapterMigrationInfo.xml is invalid! Please correct it!")
	else:	
		if (WPSMigrationUtil.validateNewRARFolder()):
			PrintHelper.printInfo("----newRarDir property in adapterMigrationInfo.xml is invalid! Please correct it!")
		else:	
			nodeType = getNodeType()
			if ( nodeType=="dmgr" ):
				if ( '' == getClusterName() ):
					PrintHelper.printInfo("----DMGR migration...")
				else:
					PrintHelper.printInfo("----Cluster migration... Cluster="+getClusterName())
			else:
				PrintHelper.printInfo("----Node profile migration... Node="+MigrateApp.getNodename())

				# migrate embeded adapters and applications
	
			PrintHelper.printInfo("----Migrating embeded adapters and applications...")
			migrateEmbededRAApps(configurationFile)
			PrintHelper.printInfo("----Done embeded adapters and applications migration\n")

			#migrate stand-alone adapters and applications
			if ( nodeType=="dmgr" ):
				PrintHelper.printInfo("----Migrating cluster level standalone adapters and applications on Cluster...")
				migrateClusterStandaloneRAApps(configurationFile)
				PrintHelper.printInfo("----Done cluster level standalone adapters and applications migration...")
			else:
				nodeName = MigrateApp.getNodename();
				PrintHelper.printInfo("----Migrating standalone adapters and applications on node="+nodeName+"...")
				migrateStandAloneRAApps(configurationFile, nodeName)
				PrintHelper.printInfo("----Done standalone adapters and applications migration on node="+nodeName+"...\n")

			PrintHelper.printInfo("----Profile migration is done!")
#end of main program